############################################
## Classification matrix given the groups ##
############################################

class <- function(groups,k){
  
  n <- length(groups)
  z <- array(0,c(n,k),dimnames=list(1:n,paste("comp",1:k,sep="")))
  for(i in 1:n) 
    z[i,groups[i]] <- 1
  return(z)
  
}

#########################
## Polynomial function ##
#########################

polyfun <- function(x,par){
  
  r <- length(par)-1
  y <- 0
  for(i in 0:r) 
    y <- y + par[i+1]*x^i
  
  return(y)
  
}

######################
## Gaussian case: X ## 
######################

GaussianX <- function(X,weights){ 
  
  # X: a numerical vector for the unique covariate
  # weights: a numerical vector of weights the same length as X giving the weights to use for elements of X.
  
  mu.obs    <- weighted.mean(x=X,w=weights)
  sigma.obs <- sqrt(sum(weights*(X-mu.obs)^2)/sum(weights))
  l         <- sum(weights*dnorm(x=X, mean = mu.obs, sd = sigma.obs, log = TRUE))
    
  return(
    list(
      loglik = l,
      muX    = mu.obs,
      sdX    = sigma.obs
    )
  )  
  
}

########################
## Gaussian case: Y|x ## 
########################

GaussianY <- function(Y,X,r,weights,method="Nelder-Mead",start=FALSE,initial.values=NULL,maxit=1000,reltol=1e-15,trace=0){ 
  
  # Y: a numerical vector for the response variable
  # X: a numerical vector for the unique covariate
  # r: the polynomial order
  # weights: a numerical vector of weights the same length as X giving the weights to use for elements of X.
  
  if(start==FALSE){
    
    modelY      <- lm(Y ~ poly(X, r, raw=TRUE),weights=weights) 
    beta.obs    <- modelY$coefficients
    sigma.obs   <- summary(modelY)$sigma
    
    starting.values <- c(beta.obs,log(sigma.obs))
    
  }
  
  f <- function(par,Y,X,r,weights){
    
    sigma <- exp(par[r+2])
    
    l <- -sum(weights*dnorm(x=Y, mean = polyfun(X,par[1:(r+1)]), sd = sigma, log = TRUE))
    
  }
  
  if(start==FALSE)
    res <- optim(par=starting.values, fn=f, Y=Y, X=X, r=r, weights=weights, method=method, control = list(maxit=maxit,reltol=reltol,trace=trace))
  if(start==TRUE)
    res <- optim(par=initial.values, fn=f, Y=Y, X=X, r=r, weights=weights, method=method, control = list(maxit=maxit,reltol=reltol,trace=trace))
  
  loglik  <- -res$value
  est     <-  res$par
  
  beta.hat  <- est[1:(r+1)]
  sigma.hat <- exp(est[r+2])
  
  return(
    list(
      loglik = loglik,
      beta  = beta.hat,
      sdY   = sigma.hat
    )
  )  
  
}



